@@ -26,7 +26,7 @@ class CachedDoubler extends Doubler
      */
     private static $classes = array();
 
-    protected function createDoubleClass(ReflectionClass $class = null, array $interfaces)
+    protected function createDoubleClass(?ReflectionClass $class, array $interfaces)
     {
         $classId = $this->generateClassId($class, $interfaces);
         if (isset(self::$classes[$classId])) {
@@ -42,7 +42,7 @@ class CachedDoubler extends Doubler
      *
      * @return string
      */
-    private function generateClassId(ReflectionClass $class = null, array $interfaces)
+    private function generateClassId(?ReflectionClass $class, array $interfaces)
     {
         $parts = array();
         if (null !== $class) {
