@@ -17,18 +17,19 @@ use Prophecy\Exception\Prophecy\MethodProphecyException;
 
 class UnexpectedCallsException extends MethodProphecyException implements PredictionException
 {
-    private $calls = array();
+    /** @var list<Call> */
+    private $calls;
 
     /**
      * @param string         $message
      * @param MethodProphecy $methodProphecy
-     * @param list<Call>     $calls
+     * @param array<Call>     $calls
      */
     public function __construct($message, MethodProphecy $methodProphecy, array $calls)
     {
         parent::__construct($message, $methodProphecy);
 
-        $this->calls = $calls;
+        $this->calls = array_values($calls);
     }
 
     /**
