@@ -26,9 +26,9 @@ class NoCallsPrediction implements PredictionInterface
 {
     private $util;
 
-    public function __construct(StringUtil $util = null)
+    public function __construct(?StringUtil $util = null)
     {
-        $this->util = $util ?: new StringUtil;
+        $this->util = $util ?: new StringUtil();
     }
 
     public function check(array $calls, ObjectProphecy $object, MethodProphecy $method)
@@ -40,9 +40,9 @@ class NoCallsPrediction implements PredictionInterface
         $verb = count($calls) === 1 ? 'was' : 'were';
 
         throw new UnexpectedCallsException(sprintf(
-            "No calls expected that match:\n".
-            "  %s->%s(%s)\n".
-            "but %d %s made:\n%s",
+            "No calls expected that match:\n"
+            ."  %s->%s(%s)\n"
+            ."but %d %s made:\n%s",
             get_class($object->reveal()),
             $method->getMethodName(),
             $method->getArgumentsWildcard(),
