@@ -2,16 +2,15 @@
 
 namespace Tests\Prophecy\Comparator;
 
+use PHPUnit\Framework\Attributes\Test;
 use Prophecy\Comparator\ClosureComparator;
 use Prophecy\Comparator\FactoryProvider;
 use PHPUnit\Framework\TestCase;
 
 class FactoryProviderTest extends TestCase
 {
-    /**
-     * @test
-     */
-    function it_should_have_ClosureComparator_registered()
+    #[Test]
+    function it_should_have_ClosureComparator_registered(): void
     {
         $comparator = FactoryProvider::getInstance()->getComparatorFor(function () {}, function () {});
 
