@@ -2,6 +2,7 @@
 
 namespace Tests\Prophecy\Doubler\ClassPatch;
 
+use PHPUnit\Framework\Attributes\Test;
 use PHPUnit\Framework\TestCase;
 use Prophecy\Doubler\ClassPatch\MagicCallPatch;
 use Prophecy\Doubler\Generator\ClassMirror;
@@ -10,10 +11,8 @@ use Prophecy\Doubler\Generator\Node\ClassNode;
 
 class MagicCallPatchTest extends TestCase
 {
-    /**
-     * @test
-     */
-    public function it_supports_classes_with_invalid_tags()
+    #[Test]
+    public function it_supports_classes_with_invalid_tags(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithPhpdocClass');
 
@@ -29,10 +28,8 @@ class MagicCallPatchTest extends TestCase
         $this->assertTrue(true);
     }
 
-    /**
-     * @test
-     */
-    public function it_supports_arguments_for_magic_methods()
+    #[Test]
+    public function it_supports_arguments_for_magic_methods(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithPhpdocClass');
 
