@@ -3,20 +3,24 @@
 namespace Tests\Prophecy\Doubler\Generator;
 
 use Fixtures\Prophecy\SelfReferencing;
+use PHPUnit\Framework\Attributes\Test;
 use PHPUnit\Framework\TestCase;
 use Prophecy\Doubler\Generator\ClassMirror;
 use Prophecy\Doubler\Generator\Node\ArgumentTypeNode;
 use Prophecy\Doubler\Generator\Node\ReturnTypeNode;
+use Prophecy\Doubler\Generator\Node\Type\BuiltinType;
+use Prophecy\Doubler\Generator\Node\Type\IntersectionType;
+use Prophecy\Doubler\Generator\Node\Type\ObjectType;
+use Prophecy\Doubler\Generator\Node\Type\SimpleType;
+use Prophecy\Doubler\Generator\Node\Type\UnionType;
 use Prophecy\Exception\Doubler\ClassMirrorException;
 use Prophecy\Exception\InvalidArgumentException;
 use Prophecy\Prophet;
 
 class ClassMirrorTest extends TestCase
 {
-    /**
-     * @test
-     */
-    public function it_reflects_allowed_magic_methods()
+    #[Test]
+    public function it_reflects_allowed_magic_methods(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\SpecialMethods');
 
@@ -27,10 +31,8 @@ class ClassMirrorTest extends TestCase
         $this->assertCount(7, $node->getMethods());
     }
 
-    /**
-     * @test
-     */
-    public function it_reflects_protected_abstract_methods()
+    #[Test]
+    public function it_reflects_protected_abstract_methods(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithProtectedAbstractMethod');
 
@@ -46,10 +48,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals('protected', $methodNodes['innerDetail']->getVisibility());
     }
 
-    /**
-     * @test
-     */
-    public function it_reflects_public_static_methods()
+    #[Test]
+    public function it_reflects_public_static_methods(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithStaticMethod');
 
@@ -65,10 +65,8 @@ class ClassMirrorTest extends TestCase
         $this->assertTrue($methodNodes['innerDetail']->isStatic());
     }
 
-    /**
-     * @test
-     */
-    public function it_marks_required_args_without_types_as_not_optional()
+    #[Test]
+    public function it_marks_required_args_without_types_as_not_optional(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithArguments');
 
@@ -88,10 +86,8 @@ class ClassMirrorTest extends TestCase
         $this->assertFalse($argNodes[0]->isVariadic());
     }
 
-    /**
-     * @test
-     */
-    public function it_properly_reads_methods_arguments_with_types()
+    #[Test]
+    public function it_properly_reads_methods_arguments_with_types(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithArguments');
 
@@ -105,28 +101,29 @@ class ClassMirrorTest extends TestCase
 
 
         $this->assertEquals('arg_1', $argNodes[0]->getName());
-        $this->assertEquals(new ArgumentTypeNode('ArrayAccess'), $argNodes[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new ObjectType('ArrayAccess')), $argNodes[0]->getTypeNode());
         $this->assertFalse($argNodes[0]->isOptional());
 
         $this->assertEquals('arg_2', $argNodes[1]->getName());
-        $this->assertEquals(new ArgumentTypeNode('array'), $argNodes[1]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new BuiltinType('array')), $argNodes[1]->getTypeNode());
         $this->assertTrue($argNodes[1]->isOptional());
         $this->assertEquals(array(), $argNodes[1]->getDefault());
         $this->assertFalse($argNodes[1]->isPassedByReference());
         $this->assertFalse($argNodes[1]->isVariadic());
 
         $this->assertEquals('arg_3', $argNodes[2]->getName());
-        $this->assertEquals(new ArgumentTypeNode('ArrayAccess', 'null'), $argNodes[2]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new UnionType([
+            new BuiltinType('null'),
+            new ObjectType('ArrayAccess'),
+        ])), $argNodes[2]->getTypeNode());
         $this->assertTrue($argNodes[2]->isOptional());
         $this->assertNull($argNodes[2]->getDefault());
         $this->assertFalse($argNodes[2]->isPassedByReference());
         $this->assertFalse($argNodes[2]->isVariadic());
     }
 
-    /**
-     * @test
-     */
-    public function it_properly_reads_methods_arguments_with_callable_types()
+    #[Test]
+    public function it_properly_reads_methods_arguments_with_callable_types(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithCallableArgument');
 
@@ -139,23 +136,24 @@ class ClassMirrorTest extends TestCase
         $this->assertCount(2, $argNodes);
 
         $this->assertEquals('arg_1', $argNodes[0]->getName());
-        $this->assertEquals(new ArgumentTypeNode('callable'), $argNodes[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new BuiltinType('callable')), $argNodes[0]->getTypeNode());
         $this->assertFalse($argNodes[0]->isOptional());
         $this->assertFalse($argNodes[0]->isPassedByReference());
         $this->assertFalse($argNodes[0]->isVariadic());
 
         $this->assertEquals('arg_2', $argNodes[1]->getName());
-        $this->assertEquals(new ArgumentTypeNode('callable', 'null'), $argNodes[1]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new UnionType([
+            new BuiltinType('null'),
+            new BuiltinType('callable'),
+        ])), $argNodes[1]->getTypeNode());
         $this->assertTrue($argNodes[1]->isOptional());
         $this->assertNull($argNodes[1]->getDefault());
         $this->assertFalse($argNodes[1]->isPassedByReference());
         $this->assertFalse($argNodes[1]->isVariadic());
     }
 
-    /**
-     * @test
-     */
-    public function it_properly_reads_methods_variadic_arguments()
+    #[Test]
+    public function it_properly_reads_methods_variadic_arguments(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithVariadicArgument');
 
@@ -174,10 +172,8 @@ class ClassMirrorTest extends TestCase
         $this->assertTrue($argNodes[0]->isVariadic());
     }
 
-    /**
-     * @test
-     */
-    public function it_properly_reads_methods_typehinted_variadic_arguments()
+    #[Test]
+    public function it_properly_reads_methods_typehinted_variadic_arguments(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithTypehintedVariadicArgument');
 
@@ -190,16 +186,14 @@ class ClassMirrorTest extends TestCase
         $this->assertCount(1, $argNodes);
 
         $this->assertEquals('args', $argNodes[0]->getName());
-        $this->assertEquals(new ArgumentTypeNode('array'), $argNodes[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new BuiltinType('array')), $argNodes[0]->getTypeNode());
         $this->assertFalse($argNodes[0]->isOptional());
         $this->assertFalse($argNodes[0]->isPassedByReference());
         $this->assertTrue($argNodes[0]->isVariadic());
     }
 
-    /**
-     * @test
-     */
-    public function it_marks_passed_by_reference_args_as_passed_by_reference()
+    #[Test]
+    public function it_marks_passed_by_reference_args_as_passed_by_reference(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithReferences');
 
@@ -217,10 +211,8 @@ class ClassMirrorTest extends TestCase
         $this->assertTrue($argNodes[1]->isPassedByReference());
     }
 
-    /**
-     * @test
-     */
-    public function it_throws_an_exception_if_class_is_final()
+    #[Test]
+    public function it_throws_an_exception_if_class_is_final(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\FinalClass');
 
@@ -231,10 +223,8 @@ class ClassMirrorTest extends TestCase
         $mirror->reflect($class, array());
     }
 
-    /**
-     * @test
-     */
-    public function it_ignores_final_methods()
+    #[Test]
+    public function it_ignores_final_methods(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithFinalMethod');
 
@@ -245,10 +235,8 @@ class ClassMirrorTest extends TestCase
         $this->assertCount(0, $classNode->getMethods());
     }
 
-    /**
-     * @test
-     */
-    public function it_marks_final_methods_as_unextendable()
+    #[Test]
+    public function it_marks_final_methods_as_unextendable(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithFinalMethod');
 
@@ -260,10 +248,8 @@ class ClassMirrorTest extends TestCase
         $this->assertFalse($classNode->isExtendable('finalImplementation'));
     }
 
-    /**
-     * @test
-     */
-    public function it_throws_an_exception_if_interface_provided_instead_of_class()
+    #[Test]
+    public function it_throws_an_exception_if_interface_provided_instead_of_class(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\EmptyInterface');
 
@@ -274,10 +260,8 @@ class ClassMirrorTest extends TestCase
         $mirror->reflect($class, array());
     }
 
-    /**
-     * @test
-     */
-    public function it_reflects_all_interfaces_methods()
+    #[Test]
+    public function it_reflects_all_interfaces_methods(): void
     {
         $mirror = new ClassMirror();
 
@@ -299,10 +283,8 @@ class ClassMirrorTest extends TestCase
         $this->assertTrue($classNode->hasMethod('getVisibility'));
     }
 
-    /**
-     * @test
-     */
-    public function it_ignores_virtually_private_methods()
+    #[Test]
+    public function it_ignores_virtually_private_methods(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithVirtuallyPrivateMethod');
 
@@ -316,10 +298,8 @@ class ClassMirrorTest extends TestCase
         $this->assertFalse($classNode->hasMethod('_getName'));
     }
 
-    /**
-     * @test
-     */
-    public function it_does_not_throw_exception_for_virtually_private_finals()
+    #[Test]
+    public function it_does_not_throw_exception_for_virtually_private_finals(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithFinalVirtuallyPrivateMethod');
 
@@ -330,10 +310,8 @@ class ClassMirrorTest extends TestCase
         $this->assertCount(0, $classNode->getMethods());
     }
 
-    /**
-     * @test
-     */
-    public function it_reflects_return_typehints()
+    #[Test]
+    public function it_reflects_return_typehints(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\WithReturnTypehints');
 
@@ -346,15 +324,13 @@ class ClassMirrorTest extends TestCase
         $this->assertTrue($classNode->hasMethod('getSelf'));
         $this->assertTrue($classNode->hasMethod('getParent'));
 
-        $this->assertEquals(new ReturnTypeNode('string'), $classNode->getMethod('getName')->getReturnTypeNode());
-        $this->assertEquals(new ReturnTypeNode('\Fixtures\Prophecy\WithReturnTypehints'), $classNode->getMethod('getSelf')->getReturnTypeNode());
-        $this->assertEquals(new ReturnTypeNode('\Fixtures\Prophecy\EmptyClass'), $classNode->getMethod('getParent')->getReturnTypeNode());
+        $this->assertEquals(new ReturnTypeNode(new BuiltinType('string')), $classNode->getMethod('getName')->getReturnTypeNode());
+        $this->assertEquals(new ReturnTypeNode(new ObjectType('Fixtures\Prophecy\WithReturnTypehints')), $classNode->getMethod('getSelf')->getReturnTypeNode());
+        $this->assertEquals(new ReturnTypeNode(new ObjectType('Fixtures\Prophecy\EmptyClass')), $classNode->getMethod('getParent')->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_throws_an_exception_if_class_provided_in_interfaces_list()
+    #[Test]
+    public function it_throws_an_exception_if_class_provided_in_interfaces_list(): void
     {
         $class = new \ReflectionClass('Fixtures\Prophecy\EmptyClass');
 
@@ -365,10 +341,8 @@ class ClassMirrorTest extends TestCase
         $mirror->reflect(null, array($class));
     }
 
-    /**
-     * @test
-     */
-    public function it_throws_an_exception_if_not_reflection_provided_as_interface()
+    #[Test]
+    public function it_throws_an_exception_if_not_reflection_provided_as_interface(): void
     {
         $mirror = new ClassMirror();
 
@@ -377,30 +351,8 @@ class ClassMirrorTest extends TestCase
         $mirror->reflect(null, array(null));
     }
 
-    /**
-     * @test
-     */
-    public function it_doesnt_use_scalar_typehints()
-    {
-        if (PHP_VERSION_ID >= 80000) {
-            $this->markTestSkipped('Internal types have scalar hints in PHP 8');
-        }
-
-        $mirror = new ClassMirror();
-
-        $classNode = $mirror->reflect(new \ReflectionClass('ReflectionMethod'), array());
-        $method = $classNode->getMethod('export');
-        $arguments = $method->getArguments();
-
-        $this->assertEquals(new ArgumentTypeNode(), $arguments[0]->getTypeNode());
-        $this->assertEquals(new ArgumentTypeNode(), $arguments[1]->getTypeNode());
-        $this->assertEquals(new ArgumentTypeNode(), $arguments[2]->getTypeNode());
-    }
-
-    /**
-     * @test
-     */
-    public function it_doesnt_fail_to_typehint_nonexistent_FQCN()
+    #[Test]
+    public function it_doesnt_fail_to_typehint_nonexistent_FQCN(): void
     {
         $mirror = new ClassMirror();
 
@@ -410,23 +362,22 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ArgumentTypeNode('I\Simply\Am\Nonexistent'), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_doesnt_fail_on_array_nullable_parameter_with_not_null_default_value()
+    #[Test]
+    public function it_doesnt_fail_on_array_nullable_parameter_with_not_null_default_value(): void
     {
         $mirror = new ClassMirror();
 
         $classNode = $mirror->reflect(new \ReflectionClass('Fixtures\Prophecy\NullableArrayParameter'), array());
         $method = $classNode->getMethod('iHaveNullableArrayParameterWithNotNullDefaultValue');
         $arguments = $method->getArguments();
-        $this->assertEquals(new ArgumentTypeNode('array', 'null'), $arguments[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new UnionType([
+            new BuiltinType('null'),
+            new BuiltinType('array'),
+        ])), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_doesnt_fail_to_typehint_nonexistent_RQCN()
+    #[Test]
+    public function it_doesnt_fail_to_typehint_nonexistent_RQCN(): void
     {
         $mirror = new ClassMirror();
 
@@ -436,10 +387,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ArgumentTypeNode('I\Simply\Am\Not'), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    function it_doesnt_fail_when_method_is_extended_with_more_params()
+    #[Test]
+    function it_doesnt_fail_when_method_is_extended_with_more_params(): void
     {
         $mirror = new ClassMirror();
 
@@ -454,10 +403,8 @@ class ClassMirrorTest extends TestCase
         $this->assertCount(2, $method->getArguments());
     }
 
-    /**
-     * @test
-     */
-    function it_doesnt_fail_to_mock_self_referencing_interface()
+    #[Test]
+    function it_doesnt_fail_to_mock_self_referencing_interface(): void
     {
         $mirror = new ClassMirror();
 
@@ -471,10 +418,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode(SelfReferencing::class), $method->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    function it_changes_argument_names_if_they_are_varying()
+    #[Test]
+    function it_changes_argument_names_if_they_are_varying(): void
     {
         // Use test doubles in this test, as arguments named ... in the Reflection API can only happen for internal classes
         $prophet = new Prophet();
@@ -526,8 +471,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals('__dot_dot_dot__', $argumentNode->getName());
     }
 
-    /** @test */
-    public function it_can_double_a_class_with_union_return_types()
+    #[Test]
+    public function it_can_double_a_class_with_union_return_types(): void
     {
         if (PHP_VERSION_ID < 80000) {
             $this->markTestSkipped('Union types are not supported in this PHP version');
@@ -536,13 +481,15 @@ class ClassMirrorTest extends TestCase
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\UnionReturnTypes'), []);
         $methodNode = $classNode->getMethods()['doSomething'];
 
-        $this->assertSame(['\stdClass', 'bool'], $methodNode->getReturnTypeNode()->getTypes());
+
+        $this->assertEquals(new UnionType([
+            new ObjectType('stdClass'),
+            new BuiltinType('bool'),
+        ]), $methodNode->getReturnTypeNode()->getType());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_class_with_union_return_type_with_false()
+    #[Test]
+    public function it_can_double_a_class_with_union_return_type_with_false(): void
     {
         if (PHP_VERSION_ID < 80000) {
             $this->markTestSkipped('Union types with false are not supported in this PHP version');
@@ -551,11 +498,14 @@ class ClassMirrorTest extends TestCase
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\UnionReturnTypeFalse'), []);
         $methodNode = $classNode->getMethods()['method'];
 
-        $this->assertSame(['\stdClass', 'false'], $methodNode->getReturnTypeNode()->getTypes());
+        $this->assertEquals(new UnionType([
+            new ObjectType('stdClass'),
+            new BuiltinType('false'),
+        ]), $methodNode->getReturnTypeNode()->getType());
     }
 
-    /** @test */
-    public function it_can_double_a_class_with_union_argument_types()
+    #[Test]
+    public function it_can_double_a_class_with_union_argument_types(): void
     {
         if (PHP_VERSION_ID < 80000) {
             $this->markTestSkipped('Union types are not supported in this PHP version');
@@ -564,13 +514,11 @@ class ClassMirrorTest extends TestCase
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\UnionArgumentTypes'), []);
         $methodNode = $classNode->getMethods()['doSomething'];
 
-        $this->assertEquals(new ArgumentTypeNode('bool', '\\stdClass'), $methodNode->getArguments()[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(\stdClass::class, 'bool'), $methodNode->getArguments()[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_class_with_union_argument_type_with_false()
+    #[Test]
+    public function it_can_double_a_class_with_union_argument_type_with_false(): void
     {
         if (PHP_VERSION_ID < 80000) {
             $this->markTestSkipped('Union types with false are not supported in this PHP version');
@@ -579,11 +527,11 @@ class ClassMirrorTest extends TestCase
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\UnionArgumentTypeFalse'), []);
         $methodNode = $classNode->getMethods()['method'];
 
-        $this->assertEquals(new ArgumentTypeNode('false', '\stdClass'), $methodNode->getArguments()[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(\stdClass::class, 'false'), $methodNode->getArguments()[0]->getTypeNode());
     }
 
-    /** @test */
-    public function it_can_double_a_class_with_mixed_types()
+    #[Test]
+    public function it_can_double_a_class_with_mixed_types(): void
     {
         if (PHP_VERSION_ID < 80000) {
             $this->markTestSkipped('Mixed type is not supported in this PHP version');
@@ -596,8 +544,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('mixed'), $methodNode->getReturnTypeNode());
     }
 
-    /** @test */
-    public function it_can_double_inherited_self_return_type()
+    #[Test]
+    public function it_can_double_inherited_self_return_type(): void
     {
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\ClassExtendAbstractWithMethodWithReturnType'), []);
         $methodNode = $classNode->getMethods()['returnSelf'];
@@ -605,10 +553,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('Fixtures\Prophecy\AbstractBaseClassWithMethodWithReturnType'), $methodNode->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_never_return_type()
+    #[Test]
+    public function it_can_double_never_return_type(): void
     {
         if (PHP_VERSION_ID < 80100) {
             $this->markTestSkipped('Never type is not supported in this PHP version');
@@ -621,10 +567,8 @@ class ClassMirrorTest extends TestCase
 
     }
 
-    /**
-     * @test
-     */
-    public function it_can_not_double_an_enum()
+    #[Test]
+    public function it_can_not_double_an_enum(): void
     {
         if (PHP_VERSION_ID < 80100) {
             $this->markTestSkipped('Enums are not supported in this PHP version');
@@ -635,38 +579,50 @@ class ClassMirrorTest extends TestCase
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\Enum'), []);
     }
 
-    /**
-     * @test
-     */
-    public function it_can_not_double_intersection_return_types()
+    #[Test]
+    public function it_can_double_intersection_return_types(): void
     {
         if (PHP_VERSION_ID < 80100) {
             $this->markTestSkipped('Intersection types are not supported in this PHP version');
         }
 
-        $this->expectException(ClassMirrorException::class);
-
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\IntersectionReturnType'), []);
+
+        $method = $classNode->getMethod('doSomething');
+        $returnType = $method->getReturnTypeNode();
+
+        $this->assertEquals(
+            new IntersectionType([
+                new ObjectType('Fixtures\Prophecy\Bar'),
+                new ObjectType('Fixtures\Prophecy\Baz'),
+            ]),
+            $returnType->getType()
+        );
     }
 
-    /**
-     * @test
-     */
-    public function it_can_not_double_intersection_argument_types()
+    #[Test]
+    public function it_can_double_intersection_argument_types(): void
     {
         if (PHP_VERSION_ID < 80100) {
             $this->markTestSkipped('Intersection types are not supported in this PHP version');
         }
 
-        $this->expectException(ClassMirrorException::class);
-
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\IntersectionArgumentType'), []);
+
+        $method = $classNode->getMethod('doSomething');
+        $argType = $method->getArguments()[0]->getTypeNode();
+
+        $this->assertEquals(
+            new IntersectionType([
+                new ObjectType('Fixtures\Prophecy\Bar'),
+                new ObjectType('Fixtures\Prophecy\Baz'),
+            ]),
+            $argType->getType()
+        );
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_standalone_return_type_of_false()
+    #[Test]
+    public function it_can_double_a_standalone_return_type_of_false(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Standalone return type of false is not supported in this PHP version');
@@ -678,10 +634,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('false'), $methodNode->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_standalone_parameter_type_of_false()
+    #[Test]
+    public function it_can_double_a_standalone_parameter_type_of_false(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Standalone parameter type of false is not supported in this PHP version');
@@ -694,10 +648,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ArgumentTypeNode('false'), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_nullable_return_type_of_false()
+    #[Test]
+    public function it_can_double_a_nullable_return_type_of_false(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Nullable return type of false is not supported in this PHP version');
@@ -709,10 +661,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('null', 'false'), $methodNode->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_nullable_parameter_type_of_false()
+    #[Test]
+    public function it_can_double_a_nullable_parameter_type_of_false(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Nullable parameter type of false is not supported in this PHP version');
@@ -722,41 +672,59 @@ class ClassMirrorTest extends TestCase
         $method = $classNode->getMethod('method');
         $arguments = $method->getArguments();
 
-        $this->assertEquals(new ArgumentTypeNode('null', 'false'), $arguments[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new UnionType([
+            new BuiltinType('null'),
+            new BuiltinType('false'),
+        ])), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_not_double_dnf_intersection_argument_types()
+    #[Test]
+    public function it_can_not_double_dnf_intersection_argument_types(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('DNF intersection types are not supported in this PHP version');
         }
 
-        $this->expectException(ClassMirrorException::class);
-
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\DnfArgumentType'), []);
+
+
+        $method = $classNode->getMethod('doSomething');
+        $argType = $method->getArguments()[0]->getTypeNode();
+
+        $this->assertEquals(
+            new UnionType([
+                new IntersectionType([
+                    new ObjectType('Fixtures\Prophecy\A'),
+                    new ObjectType('Fixtures\Prophecy\B'),
+                ]),
+                new ObjectType('Fixtures\Prophecy\C'),
+            ]),
+            $argType->getType()
+        );
     }
 
-    /**
-     * @test
-     */
-    public function it_can_not_double_dnf_intersection_return_types()
+    #[Test]
+    public function it_can_double_dnf_intersection_return_types(): void
     {
-        if (PHP_VERSION_ID < 80200) {
-            $this->markTestSkipped('DNF intersection types are not supported in this PHP version');
-        }
-
-        $this->expectException(ClassMirrorException::class);
-
         $classNode = (new ClassMirror())->reflect(new \ReflectionClass('Fixtures\Prophecy\DnfReturnType'), []);
+
+        $method = $classNode->getMethod('doSomething');
+        $returnType = $method->getReturnTypeNode();
+
+        $this->assertEquals(
+            new UnionType([
+                new IntersectionType([
+                    new ObjectType('Fixtures\Prophecy\A'),
+                    new ObjectType('Fixtures\Prophecy\B'),
+                ]),
+                new ObjectType('Fixtures\Prophecy\C'),
+            ]),
+            $returnType->getType()
+        );
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_standalone_return_type_of_true()
+    #[Test]
+    public function it_can_double_a_standalone_return_type_of_true(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Standalone return type of true is not supported in this PHP version');
@@ -768,10 +736,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('true'), $methodNode->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_reflects_non_read_only_class()
+    #[Test]
+    public function it_reflects_non_read_only_class(): void
     {
         $classNode = (new ClassMirror())->reflect(
             new \ReflectionClass('Fixtures\Prophecy\EmptyClass'),
@@ -781,10 +747,8 @@ class ClassMirrorTest extends TestCase
         $this->assertFalse($classNode->isReadOnly());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_standalone_parameter_type_of_true()
+    #[Test]
+    public function it_can_double_a_standalone_parameter_type_of_true(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Standalone parameter type of true is not supported in this PHP version');
@@ -797,10 +761,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ArgumentTypeNode('true'), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_nullable_return_type_of_true()
+    #[Test]
+    public function it_can_double_a_nullable_return_type_of_true(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Nullable return type of true is not supported in this PHP version');
@@ -812,10 +774,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('null', 'true'), $methodNode->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_nullable_parameter_type_of_true()
+    #[Test]
+    public function it_can_double_a_nullable_parameter_type_of_true(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Nullable parameter type of true is not supported in this PHP version');
@@ -825,13 +785,14 @@ class ClassMirrorTest extends TestCase
         $method = $classNode->getMethod('method');
         $arguments = $method->getArguments();
 
-        $this->assertEquals(new ArgumentTypeNode('null', 'true'), $arguments[0]->getTypeNode());
+        $this->assertEquals(new ArgumentTypeNode(new UnionType([
+            new BuiltinType('null'),
+            new BuiltinType('true'),
+        ])), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_standalone_return_type_of_null()
+    #[Test]
+    public function it_can_double_a_standalone_return_type_of_null(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Standalone return type of null is not supported in this PHP version');
@@ -843,10 +804,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ReturnTypeNode('null'), $methodNode->getReturnTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_can_double_a_standalone_parameter_type_of_null()
+    #[Test]
+    public function it_can_double_a_standalone_parameter_type_of_null(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Standalone parameter type of null is not supported in this PHP version');
@@ -859,10 +818,8 @@ class ClassMirrorTest extends TestCase
         $this->assertEquals(new ArgumentTypeNode('null'), $arguments[0]->getTypeNode());
     }
 
-    /**
-     * @test
-     */
-    public function it_reflects_read_only_class()
+    #[Test]
+    public function it_reflects_read_only_class(): void
     {
         if (PHP_VERSION_ID < 80200) {
             $this->markTestSkipped('Read only classes are not supported in this PHP version');
